using System;
using System.Data;
using System.Data.OleDb;


namespace VfpConsoleApp
{
	/// <summary>
	/// Class CMain - containes the main entry point for the application
	/// </summary>
	class CMain
	{
		/// <summary>
		/// void DisplayUsage() - Display help command line
		/// </summary>
		/// <returns> No return value. </returns>
		static void DisplayUsage()
		{
			// User did not provide enough parameters. 
			// Display usage. 
			Console.WriteLine("ConsoleApp: example of printing VFP-table into system console."); 
			Console.WriteLine("--- Usage: ConsoleApp.exe"); 
			Console.WriteLine("[/? <- Display command line help]");
			Console.WriteLine("[/s\"Connection string\"]");
			Console.WriteLine("[/c\"SQL command\"]"); 
			Console.WriteLine("[/t\"FieldDelimiter\"]"); 
			Console.WriteLine("[/l\"EndLineDelimiter\"]");
			Console.WriteLine("--- Exaples:"); 
			Console.WriteLine("ConsoleApp.exe /t\";\" /c\"select * from products where prod_name like 'C%'\">result.txt");
			Console.WriteLine("ConsoleApp.exe /s\"Provider=SQLOLEDB.1;DSN=LocalServer;UID=michael;APP=Microsoft(R) Visual FoxPro;WSID=FOXHOLE;DATABASE=pubs;Trusted_Connection=Yes\" /c\"SELECT * FROM dbo.authors\""); 
			Console.WriteLine("--- Default values:"); 
			Console.WriteLine("/s\""+@"provider=VFPOLEDB.1;data source='C:\PROGRAM FILES\MICROSOFT VISUAL FOXPRO 7\SAMPLES\DATA\TESTDATA.DBC';password='';user id=''"+"\"");
			Console.WriteLine("/c\"SELECT * FROM employee\""); 
			Console.WriteLine("/t\"\\t\""); 
			Console.WriteLine("/l\"\\r\\n\""); 
		}

		/// <summary>
		/// The main entry point for the application.
		/// </summary>
		/// <param name="args"> See remarks.</param>
		/// <returns> No return value. </returns>
		/// <remarks> Command line parameters:<br>
		/// [/? - Display help]<br>
		/// [/s"Connection string"]<br>
		/// [/c"SQL command"]<br>
		/// [/t"FieldDelimiter"]<br>
		/// [/l"EndLineDelimiter"]
		/// </remarks>

		[STAThread]
		static void Main(string[] args)
		{
			// Create CVfpData object
			CVfpData oVfpData = new CVfpData();
			
			// Check args
			foreach (string strArg in args)
			{
				string strCurArg = strArg.Trim(); 
				if (strCurArg.CompareTo("\\h") == 0) 
				{ 
					DisplayUsage();
					return ;
				}
				if (strCurArg.CompareTo("/h") == 0) 
				{ 
					DisplayUsage();
					return ;
				}
				if (strCurArg.CompareTo("/?") == 0) 
				{ 
					DisplayUsage();
					return ;
				}
				if (strCurArg.CompareTo("\\?") == 0) 
				{ 
					DisplayUsage();
					return ;
				}
				if (strCurArg.CompareTo("-?") == 0) 
				{ 
					DisplayUsage();
					return ;
				}
				if (strCurArg.StartsWith("/s"))
				{
					oVfpData.strConnection = strCurArg.Substring(2);
				}
				if (strCurArg.StartsWith("/c"))
				{
					oVfpData.strSqlCmd = strCurArg.Substring(2);
				}
				if (strCurArg.StartsWith("/t"))
				{
					oVfpData.strFldDelimiter = strCurArg.Substring(2);
				}
				if (strCurArg.StartsWith("/l"))
				{
					oVfpData.strEndLine = strCurArg.Substring(2);
				}
			}

			// Read & Display result
			oVfpData.GetData();
		}
	}
	
	/// <summary>
	/// Class CVfpData - example of printing VFP-table into system console
	/// </summary>
	/// <remarks>You can overwrite method PrintRows(DataSet) for any other purposes</remarks>
	public class CVfpData
	{
		/// <summary>
		/// CVfpData() - default constructor
		/// </summary>
		/// <returns> No return value. </returns>
		public CVfpData()
		{
		}

		/// <summary>
		/// CVfpData(string strConnection, string strSqlCmd) - Overload constructor with parameters 
		/// </summary>
		/// <param name="strConnection"> The Connection string to data source </param>
		/// <param name="strSqlCmd"> The SQL commamd to select data from data source </param> 
		/// <remarks>Overload constructor with parameters.</remarks>
		/// <returns> No return value. </returns>

		public CVfpData(string strConnection, string strSqlCmd)
		{
			if (strConnection.Length != 0)
			{
				this.strConnection = strConnection; 
			}
			if (strSqlCmd.Length != 0)
			{
				this.strSqlCmd = strSqlCmd; 
			}
		}

		/// <summary>
		/// Connection string - class field
		/// </summary>
		/// <remarks>Has default value (@"provider=VFPOLEDB.1;data source='C:\PROGRAM FILES\MICROSOFT VISUAL FOXPRO 7\SAMPLES\DATA\TESTDATA.DBC';password='';user id=''")</remarks>
		protected string m_strConnection = @"provider=VFPOLEDB.1;data source='C:\PROGRAM FILES\MICROSOFT VISUAL FOXPRO 7\SAMPLES\DATA\TESTDATA.DBC';password='';user id=''";

		/// <summary>
		/// Connection string property
		/// </summary>
		///<value>Accesses to the Connection string value</value>
		public string strConnection
		{
			get
			{
				return m_strConnection;
			}
			set
			{
				m_strConnection = value;
			}
		}

		/// <summary>
		/// SQL command - class field
		/// </summary>
		/// <remarks>Has default value ("SELECT * FROM employee").</remarks>
		protected string m_strSqlCmd = "SELECT * FROM employee";

		/// <summary>
		/// SQL command - string property
		/// </summary>
		/// <value>Accesses to the SQL command value</value>
		public string strSqlCmd
		{
			get
			{
				return m_strSqlCmd;
			}
			set
			{
				m_strSqlCmd = value;
			}
		}

		/// <summary>
		/// Field delimiter string - class field
		/// </summary>
		/// <remarks>Has default value ("\t")</remarks>
		protected string m_strFldDelimiter = "\t";

		/// <summary>
		/// Field delimiter string property
		/// </summary>
		///<value>Accesses to the field delimiter string value</value>
		public string strFldDelimiter
		{
			get
			{
				return m_strFldDelimiter;
			}
			set
			{
				m_strFldDelimiter = RestoreSlashValue(value);
			}
		}

		/// <summary>
		/// Line end string - class field
		/// </summary>
		/// <remarks>Has default value ("\r\n")</remarks>
		protected string m_strEndLine = "\r\n";

		/// <summary>
		/// Line end string property
		/// </summary>
		///<value>Accesses to the Line end string value</value>
		public string strEndLine
		{
			get
			{
				return m_strEndLine;
			}
			set
			{
				m_strEndLine = RestoreSlashValue(value);
			}
		}

		/// <summary>
		/// string RestoreSlashValue(string strParam) method - Restore slash's value [slash + next char] in strParam as equivalent value
		/// </summary>
		/// <param name="strParam"> The parameter may be have slash as containes.</param>
		/// <returns>In the returns string each [slash + next char] replaces by equivalent value.</returns>
		protected string RestoreSlashValue(string strParam)
		{
			int nInLen = strParam.Length;
			if (nInLen == 0)
			{
				return strParam;
			}
			
			System.Text.StringBuilder strRetVal = new System.Text.StringBuilder(nInLen);
			int nStart = 0;
			int nCur = (-1);
			int nLen = 0;
			while ((nCur = strParam.IndexOf("\\", nStart)) != (-1))
			{
				if ((nLen = nCur-nStart) > 0)
				{
					strRetVal.Append(strParam.Substring(nStart, nLen));
				}
				switch (strParam[++nCur])
				{
					case 'n':
						strRetVal.Append("\n");
						nCur++;
						break;
					case 'r':
						strRetVal.Append("\r");
						nCur++;
						break;
					case 't':
						strRetVal.Append("\t");
						nCur++;
						break;
					case 'v':
						strRetVal.Append("\v");
						nCur++;
						break;
					case 'a':
						strRetVal.Append("\a");
						nCur++;
						break;
					case 'b':
						strRetVal.Append("\b");
						nCur++;
						break;
					case 'f':
						strRetVal.Append("\f");
						nCur++;
						break;
					case '\'':
						strRetVal.Append("\'");
						nCur++;
						break;
					case '\"':
						strRetVal.Append("\"");
						nCur++;
						break;
					case '\\':
						strRetVal.Append("\\");
						nCur++;
						break;
					default:
						string strMsg = "Unexpected char: '";
						strMsg += strParam.Substring(nCur-1,2);
						strMsg += "' as arg for CVfpData.RestoreSlashValue()";
						throw new Exception(strMsg);
/*
						strRetVal.Append(strParam.Substring(nStart, ++nCur-nStart));
						nCur++;
						break;
*/						
				}
				nStart = nCur;
			}
			if (nStart < nInLen)
			{
				strRetVal.Append(strParam.Substring(nStart));
			}
			return strRetVal.ToString();
		}

		/// <summary>
		/// bool GetData() method - read data from data source & print result into system console
		/// </summary>
		/// <returns> True is successful, false - otherwise.</returns>
		/// <remarks>Used: OleDbConnection, OleDbDataAdapter, DataSet objects and method PrintRows(DataSet) from this class.</remarks>
		public bool GetData()
		{
			bool bRetVal = false;
			
			// Check value of properties
			if (strConnection.Length == 0)
				return bRetVal;
			if (strSqlCmd.Length == 0)
				return bRetVal;

			OleDbConnection oCn = null;
			OleDbDataAdapter oDa = null;
			DataSet oDs = null;
			try
			{
				// Read data
				oCn = new OleDbConnection(strConnection);
				oCn.Open();
				oDa = new OleDbDataAdapter(strSqlCmd, oCn);
				oDs = new DataSet();
				oDa.Fill(oDs);
				
				// Write result
				PrintRows(oDs);
				bRetVal = true;
			}
			catch(Exception e)
			{
				Console.WriteLine(e.Message); 
			}
			finally 
			{
				// Close all data resourses
				if (oDs != null)
				{
					oDs.Dispose();
				}
				if (oDa != null)
				{
					oDa.Dispose();
				}
				if (oCn != null && oCn.State.ToString() != "Closed")
				{
					oCn.Close();
				}
			}        
			return bRetVal;
		}

		/// <summary>
		/// void PrintRows(DataSet oDs) method - print contains of DataSet into system console
		/// </summary>
		/// <param name="oDs"> The DataSet conaines data to print.</param>
		/// <returns> No return value. </returns>
		/// <remarks>You can overwrite this method for any other purposes.
		/// Pay attention this method uses m_strFldDelimiter and m_strEndLine fields of this class</remarks>
		protected virtual void PrintRows(DataSet oDs)
		{
			int nColumns = 0;
			int nColumn = 0;
			foreach(DataTable oTable in oDs.Tables)
			{
				nColumns = oTable.Columns.Count;  
				foreach(DataRow oRow in oTable.Rows)
				{
					nColumn = 0;
					foreach (DataColumn oColumn in oTable.Columns)
					{
						Console.Write(oRow[oColumn].ToString().Trim()
							+((++nColumn < nColumns) ? m_strFldDelimiter : ""));
					}
					Console.Write(m_strEndLine);
				}
			}
		}
	}
}
